#include <windows.h>
#include <mmsystem.h>

BOOL InitMixer(void);
void ReleaseMixer(void);
BOOL GetMasterVolume(int *Val);
BOOL SetMasterVolume(int Val);
BOOL UpDownMasterVolume(int dif);
BOOL GetMasterMute(BOOL *Val);
BOOL SetMasterMute(BOOL Val);
BOOL ReverseMasterMute(void);

BOOL InitMasterVolumeControl(void);
BOOL InitMasterMuteControl(void);


HMIXER		g_hMixer=NULL;
MIXERCAPS	g_mxcaps;
DWORD		g_dwMinimum,g_dwMaximum;
DWORD		g_dwVolumeControlID=-1;
DWORD		g_dwMuteControlID=-1;

BOOL InitMixer(void)
{
	if(g_hMixer) return TRUE;
	if(mixerOpen(&g_hMixer,0,0,0,MIXER_OBJECTF_MIXER)!=MMSYSERR_NOERROR) return FALSE;
	if(mixerGetDevCaps((UINT)g_hMixer, &g_mxcaps, sizeof(MIXERCAPS))!=MMSYSERR_NOERROR) return FALSE;
	return	TRUE;
}

void ReleaseMixer(void)
{
	if(g_hMixer) mixerClose(g_hMixer);
	g_hMixer=NULL;
	g_dwVolumeControlID=-1;
	g_dwMuteControlID=-1;
}

//-----------------------------------------------------------------------------
//             @@@@@@@@@@@@{[
//-----------------------------------------------------------------------------

BOOL InitMasterVolumeControl(void)
{
	MIXERLINE mxl;
	MIXERCONTROL mxc;
	MIXERLINECONTROLS mxlc;

	if(g_dwVolumeControlID != -1) return TRUE;
	if(InitMixer() == FALSE) return FALSE;

	mxl.cbStruct = sizeof(MIXERLINE);
	mxl.dwComponentType = MIXERLINE_COMPONENTTYPE_DST_SPEAKERS;
	if(mixerGetLineInfo((HMIXEROBJ)g_hMixer,&mxl,
				MIXER_OBJECTF_HMIXER | MIXER_GETLINEINFOF_COMPONENTTYPE)!=MMSYSERR_NOERROR) return FALSE;

	mxlc.cbStruct = sizeof(MIXERLINECONTROLS);
	mxlc.dwLineID = mxl.dwLineID;
	mxlc.dwControlType = MIXERCONTROL_CONTROLTYPE_VOLUME;
	mxlc.cControls = 1;
	mxlc.cbmxctrl = sizeof(MIXERCONTROL);
	mxlc.pamxctrl = &mxc;
	if(mixerGetLineControls((HMIXEROBJ)g_hMixer,&mxlc,
			   MIXER_OBJECTF_HMIXER | MIXER_GETLINECONTROLSF_ONEBYTYPE) != MMSYSERR_NOERROR) return FALSE;

	g_dwMinimum = mxc.Bounds.dwMinimum;
	g_dwMaximum = mxc.Bounds.dwMaximum;
	g_dwVolumeControlID = mxc.dwControlID;

	return TRUE;
}

BOOL GetMasterVolume(int *Val)
{
	MIXERCONTROLDETAILS_UNSIGNED mxcdVolume;
	MIXERCONTROLDETAILS mxcd;

	if(InitMasterVolumeControl() == FALSE) return FALSE;

	mxcd.cbStruct = sizeof(MIXERCONTROLDETAILS);
	mxcd.dwControlID = g_dwVolumeControlID;
	mxcd.cChannels = 1;
	mxcd.cMultipleItems = 0;
	mxcd.cbDetails = sizeof(MIXERCONTROLDETAILS_UNSIGNED);
	mxcd.paDetails = &mxcdVolume;
	if(mixerGetControlDetails((HMIXEROBJ)g_hMixer,&mxcd,
				 MIXER_OBJECTF_HMIXER | MIXER_GETCONTROLDETAILSF_VALUE)!=MMSYSERR_NOERROR) return FALSE;
	
	*Val = ((mxcdVolume.dwValue-g_dwMinimum)*100 + (g_dwMaximum-g_dwMinimum)/2)/(g_dwMaximum-g_dwMinimum);

	return TRUE;
}

BOOL SetMasterVolume(int Val)
{
	MIXERCONTROLDETAILS_UNSIGNED mxcdVolume;
	MIXERCONTROLDETAILS mxcd;

	if(InitMasterVolumeControl() == FALSE) return FALSE;

	mxcdVolume.dwValue=Val*(g_dwMaximum-g_dwMinimum)/100+g_dwMinimum;

	mxcd.cbStruct = sizeof(MIXERCONTROLDETAILS);
	mxcd.dwControlID = g_dwVolumeControlID;
	mxcd.cChannels = 1;
	mxcd.cMultipleItems = 0;
	mxcd.cbDetails = sizeof(MIXERCONTROLDETAILS_UNSIGNED);
	mxcd.paDetails = &mxcdVolume;
	if (mixerSetControlDetails((HMIXEROBJ)g_hMixer,&mxcd,
				 MIXER_OBJECTF_HMIXER | MIXER_SETCONTROLDETAILSF_VALUE)!=MMSYSERR_NOERROR) return FALSE;
	
	return TRUE;
}

BOOL UpDownMasterVolume(int dif)
{
	int	Val;

	if(GetMasterVolume(&Val)==FALSE) return FALSE;

	Val+=dif;
	if(Val>100) Val=100;
	else if(Val<0) Val=0;

	if(SetMasterVolume(Val)==FALSE) return FALSE;

	return	TRUE;
}

//-----------------------------------------------------------------------------
//             @@@@@@@@@@@@~[g
//-----------------------------------------------------------------------------

BOOL InitMasterMuteControl()
{
	MIXERLINE mxl;
	MIXERCONTROL mxc;
	MIXERLINECONTROLS mxlc;

	if(g_dwMuteControlID != -1) return TRUE;
	if(InitMixer() == FALSE) return FALSE;

	mxl.cbStruct = sizeof(MIXERLINE);
	mxl.dwComponentType = MIXERLINE_COMPONENTTYPE_DST_SPEAKERS;
	if(mixerGetLineInfo((HMIXEROBJ)g_hMixer,&mxl,
				MIXER_OBJECTF_HMIXER | MIXER_GETLINEINFOF_COMPONENTTYPE)!=MMSYSERR_NOERROR) return FALSE;

	mxlc.cbStruct = sizeof(MIXERLINECONTROLS);
	mxlc.dwLineID = mxl.dwLineID;
	mxlc.dwControlType = MIXERCONTROL_CONTROLTYPE_MUTE;
	mxlc.cControls = 1;
	mxlc.cbmxctrl = sizeof(MIXERCONTROL);
	mxlc.pamxctrl = &mxc;
	if(mixerGetLineControls((HMIXEROBJ)g_hMixer,&mxlc,
			   MIXER_OBJECTF_HMIXER | MIXER_GETLINECONTROLSF_ONEBYTYPE) != MMSYSERR_NOERROR) return FALSE;

	g_dwMuteControlID = mxc.dwControlID;

	return TRUE;
}

BOOL GetMasterMute(BOOL *Val)
{
	MIXERCONTROLDETAILS_BOOLEAN mxcdMute;
	MIXERCONTROLDETAILS mxcd;

	if(InitMasterMuteControl() == FALSE) return FALSE;

	mxcd.cbStruct = sizeof(MIXERCONTROLDETAILS);
	mxcd.dwControlID = g_dwMuteControlID;
	mxcd.cChannels = 1;
	mxcd.cMultipleItems = 0;
	mxcd.cbDetails = sizeof(MIXERCONTROLDETAILS_BOOLEAN);
	mxcd.paDetails = &mxcdMute;
	if (mixerGetControlDetails((HMIXEROBJ)g_hMixer,&mxcd,
				 MIXER_OBJECTF_HMIXER | MIXER_GETCONTROLDETAILSF_VALUE)!=MMSYSERR_NOERROR) return FALSE;
	
	*Val = mxcdMute.fValue;

	return TRUE;
}

BOOL SetMasterMute(BOOL Val)
{
	MIXERCONTROLDETAILS_BOOLEAN mxcdMute;
	MIXERCONTROLDETAILS mxcd;

	if(InitMasterMuteControl() == FALSE) return FALSE;

	mxcdMute.fValue=Val;

	mxcd.cbStruct = sizeof(MIXERCONTROLDETAILS);
	mxcd.dwControlID = g_dwMuteControlID;
	mxcd.cChannels = 1;
	mxcd.cMultipleItems = 0;
	mxcd.cbDetails = sizeof(MIXERCONTROLDETAILS_BOOLEAN);
	mxcd.paDetails = &mxcdMute;
	if (mixerSetControlDetails((HMIXEROBJ)g_hMixer,&mxcd,
				 MIXER_OBJECTF_HMIXER | MIXER_SETCONTROLDETAILSF_VALUE)!=MMSYSERR_NOERROR) return FALSE;
	
	return TRUE;
}

BOOL ReverseMasterMute(void)
{
	LONG				lVal;

	if(GetMasterMute(&lVal)==FALSE) return FALSE;
	if(SetMasterMute(!lVal)==FALSE) return FALSE;
	
	return TRUE;
}
